@@ -7,7 +7,6 @@ mod config;
 mod deploy_sets;
 mod event;
 mod metrics;
-
 #[cfg(test)]
 mod tests;
 
@@ -18,13 +17,14 @@ use std::{
     time::Duration,
 };
 
-use casper_types::PublicKey;
 pub use config::Config;
 use datasize::DataSize;
 use itertools::Itertools;
 use prometheus::{self, Registry};
 use tracing::{debug, error, info, trace, warn};
 
+use casper_types::PublicKey;
+
 use crate::{
     components::{
         consensus::{BlockContext, ClContext},
@@ -37,12 +37,12 @@ use crate::{
     types::{
         appendable_block::{AddError, AppendableBlock},
         chainspec::DeployConfig,
-        BlockPayload, Chainspec, DeployHash, DeployHeader, Timestamp,
+        BlockPayload, Chainspec, DeployHash, DeployHeader, DeployOrTransferHash, Timestamp,
     },
     NodeRng,
 };
 use deploy_sets::BlockProposerDeploySets;
-pub(crate) use event::{DeployType, Event};
+pub(crate) use event::{DeployInfo, Event};
 use metrics::BlockProposerMetrics;
 
 /// Block proposer component.
@@ -69,7 +69,7 @@ type BlockHeight = u64;
 /// A queue of contents of blocks that we know have been finalized, but we are still missing
 /// notifications about finalization of some of their ancestors. It maps block height to the
 /// deploys contained in the corresponding block.
-type FinalizationQueue = HashMap<BlockHeight, Vec<DeployHash>>;
+type FinalizationQueue = HashMap<BlockHeight, Vec<DeployOrTransferHash>>;
 
 /// A queue of requests we can't respond to yet, because we aren't up to date on finalized blocks.
 /// The key is the height of the next block we will expect to be finalized at the point when we can
@@ -86,7 +86,7 @@ enum BlockProposerState {
         pending: Vec<Event>,
         /// The deploy config from the current chainspec.
         deploy_config: DeployConfig,
-        /// The configuration, containing local settings for deploy selection
+        /// The configuration, containing local settings for deploy selection.
         local_config: Config,
     },
     /// Normal operation.
@@ -195,9 +195,10 @@ where
                 effects.extend(ready_state.handle_event(effect_builder, event));
 
                 // Update metrics after the effects have been applied.
-                self.metrics
-                    .pending_deploys
-                    .set(ready_state.sets.pending.len() as i64);
+                self.metrics.pending_deploys.set(
+                    ready_state.sets.pending_deploys.len() as i64
+                        + ready_state.sets.pending_transfers.len() as i64,
+                );
             }
         };
 
@@ -207,6 +208,7 @@ where
 
 /// State of operational block proposer.
 #[derive(DataSize, Debug)]
+#[cfg_attr(test, derive(Default))]
 struct BlockProposerReady {
     /// Set of deploys currently stored in the block proposer.
     sets: BlockProposerDeploySets,
@@ -229,7 +231,7 @@ impl BlockProposerReady {
         event: Event,
     ) -> Effects<Event>
     where
-        REv: Send + From<StateStoreRequest>,
+        REv: Send + From<StorageRequest> + From<StateStoreRequest>,
     {
         match event {
             Event::Request(BlockProposerRequest::RequestBlockPayload(request)) => {
@@ -256,8 +258,8 @@ impl BlockProposerReady {
                         .ignore()
                 }
             }
-            Event::BufferDeploy { hash, deploy_type } => {
-                self.add_deploy_or_transfer(Timestamp::now(), hash, *deploy_type);
+            Event::BufferDeploy { hash, deploy_info } => {
+                self.add_deploy(Timestamp::now(), hash, *deploy_info);
                 Effects::new()
             }
             Event::Prune => {
@@ -275,7 +277,7 @@ impl BlockProposerReady {
                 Effects::new()
             }
             Event::FinalizedBlock(block) => {
-                let deploys = block.deploys_and_transfers_iter().copied().collect_vec();
+                let deploys = block.deploys_and_transfers_iter().collect_vec();
                 let mut height = block.height();
 
                 if height > self.sets.next_finalized {
@@ -305,54 +307,64 @@ impl BlockProposerReady {
         }
     }
 
-    /// Adds a deploy to the block proposer.
-    ///
-    /// Returns `false` if the deploy has been rejected.
-    fn add_deploy_or_transfer(
+    /// Adds a deploy or a transfer to the block proposer.
+    fn add_deploy(
         &mut self,
         current_instant: Timestamp,
-        hash: DeployHash,
-        deploy_or_transfer: DeployType,
+        hash: DeployOrTransferHash,
+        deploy_info: DeployInfo,
     ) {
-        if deploy_or_transfer.header().expired(current_instant) {
+        if deploy_info.header.expired(current_instant) {
             trace!(%hash, "expired deploy rejected from the buffer");
             return;
         }
-        if self.unhandled_finalized.remove(&hash) {
-            info!(%hash,
-                "deploy was previously marked as finalized, storing header"
-            );
+        if self.unhandled_finalized.remove(hash.deploy_hash()) {
+            info!(%hash, "deploy was previously marked as finalized, storing header");
             self.sets
                 .finalized_deploys
-                .insert(hash, deploy_or_transfer.take_header());
+                .insert(hash.into(), deploy_info.header);
             return;
         }
         // only add the deploy if it isn't contained in a finalized block
-        if self.sets.finalized_deploys.contains_key(&hash) {
+        if self.sets.finalized_deploys.contains_key(hash.deploy_hash()) {
             info!(%hash, "deploy rejected from the buffer");
+            return;
+        }
+
+        if hash.is_transfer() {
+            self.sets
+                .pending_transfers
+                .insert(*hash.deploy_hash(), (deploy_info, current_instant));
         } else {
             self.sets
-                .pending
-                .insert(hash, (deploy_or_transfer, current_instant));
-            info!(%hash, "added deploy to the buffer");
+                .pending_deploys
+                .insert(*hash.deploy_hash(), (deploy_info, current_instant));
         }
+
+        info!(%hash, "added deploy to the buffer");
     }
 
     /// Notifies the block proposer that a block has been finalized.
     fn finalized_deploys<I>(&mut self, deploys: I)
     where
-        I: IntoIterator<Item = DeployHash>,
+        I: IntoIterator<Item = DeployOrTransferHash>,
     {
         for deploy_hash in deploys.into_iter() {
-            match self.sets.pending.remove(&deploy_hash) {
-                Some((deploy_type, _)) => {
-                    self.sets
-                        .finalized_deploys
-                        .insert(deploy_hash, deploy_type.take_header());
+            let (hash, remove_result) = match deploy_hash {
+                DeployOrTransferHash::Deploy(hash) => {
+                    (hash, self.sets.pending_deploys.remove(&hash))
+                }
+                DeployOrTransferHash::Transfer(hash) => {
+                    (hash, self.sets.pending_transfers.remove(&hash))
+                }
+            };
+            match remove_result {
+                Some((deploy_info, _)) => {
+                    self.sets.finalized_deploys.insert(hash, deploy_info.header);
                 }
                 // If we haven't seen this deploy before, we still need to take note of it.
-                _ => {
-                    self.unhandled_finalized.insert(deploy_hash);
+                None => {
+                    self.unhandled_finalized.insert(hash);
                 }
             }
         }
@@ -366,7 +378,7 @@ impl BlockProposerReady {
         deploys: I,
     ) -> Effects<Event>
     where
-        I: IntoIterator<Item = DeployHash>,
+        I: IntoIterator<Item = DeployOrTransferHash>,
     {
         self.finalized_deploys(deploys);
         self.sets.next_finalized = self.sets.next_finalized.max(height + 1);
@@ -393,7 +405,7 @@ impl BlockProposerReady {
     }
 
     /// Checks if a deploy's dependencies are satisfied, so the deploy is eligible for inclusion.
-    fn deps_resolved(&self, header: &DeployHeader, past_deploys: &HashSet<&DeployHash>) -> bool {
+    fn deps_resolved(&self, header: &DeployHeader, past_deploys: &HashSet<DeployHash>) -> bool {
         header
             .dependencies()
             .iter()
@@ -412,14 +424,15 @@ impl BlockProposerReady {
             .ancestor_values()
             .iter()
             .flat_map(|block_payload| block_payload.deploys_and_transfers_iter())
+            .map(DeployOrTransferHash::into)
+            .take_while(|hash| !self.contains_finalized(hash))
             .collect();
         let block_timestamp = context.timestamp();
         let mut appendable_block = AppendableBlock::new(deploy_config, block_timestamp);
 
         // We prioritize transfers over deploys, so we try to include them first.
-        for (hash, (deploy_type, received_time)) in &self.sets.pending {
-            if !deploy_type.is_transfer()
-                || !self.deps_resolved(deploy_type.header(), &past_deploys)
+        for (hash, (deploy_info, received_time)) in &self.sets.pending_transfers {
+            if !self.deps_resolved(&deploy_info.header, &past_deploys)
                 || past_deploys.contains(hash)
                 || self.contains_finalized(hash)
                 || block_timestamp.saturating_diff(*received_time) < self.local_config.deploy_delay
@@ -427,7 +440,7 @@ impl BlockProposerReady {
                 continue;
             }
 
-            if let Err(err) = appendable_block.add(*hash, deploy_type) {
+            if let Err(err) = appendable_block.add_transfer(*hash, deploy_info) {
                 match err {
                     // We added the maximum number of transfers.
                     AddError::TransferCount | AddError::GasLimit | AddError::BlockSize => break,
@@ -442,9 +455,8 @@ impl BlockProposerReady {
         }
 
         // Now we try to add other deploys to the block.
-        for (hash, (deploy_type, received_time)) in &self.sets.pending {
-            if deploy_type.is_transfer()
-                || !self.deps_resolved(deploy_type.header(), &past_deploys)
+        for (hash, (deploy_info, received_time)) in &self.sets.pending_deploys {
+            if !self.deps_resolved(&deploy_info.header, &past_deploys)
                 || past_deploys.contains(hash)
                 || self.contains_finalized(hash)
                 || block_timestamp.saturating_diff(*received_time) < self.local_config.deploy_delay
@@ -452,7 +464,7 @@ impl BlockProposerReady {
                 continue;
             }
 
-            if let Err(err) = appendable_block.add(*hash, deploy_type) {
+            if let Err(err) = appendable_block.add_deploy(*hash, deploy_info) {
                 match err {
                     // We added the maximum number of deploys.
                     AddError::DeployCount => break,
