@@ -51,7 +51,7 @@ use crate::{
     reactor::ReactorExit,
     types::{
         chainspec::{Error, ProtocolConfig, CHAINSPEC_NAME},
-        ActivationPoint, Block, BlockHash, BlockHeader, Chainspec, ChainspecInfo, ExitCode,
+        ActivationPoint, Block, BlockHeader, Chainspec, ChainspecInfo, ExitCode,
     },
     utils::{self, Loadable},
     NodeRng,
@@ -176,6 +176,7 @@ pub struct ChainspecLoader {
     initial_state_root_hash: Digest,
     next_upgrade: Option<NextUpgrade>,
     initial_block: Option<Block>,
+    after_upgrade: bool,
 }
 
 impl ChainspecLoader {
@@ -214,14 +215,27 @@ impl ChainspecLoader {
         P: AsRef<Path>,
         REv: From<Event> + From<StorageRequest> + From<StateStoreRequest> + Send,
     {
-        chainspec.validate_config();
         let root_dir = chainspec_dir
             .as_ref()
             .parent()
+            .map(|path| path.to_path_buf())
             .unwrap_or_else(|| {
-                panic!("chainspec dir must have a parent");
-            })
-            .to_path_buf();
+                error!("chainspec dir must have a parent");
+                PathBuf::new()
+            });
+
+        if !chainspec.is_valid() || root_dir.as_os_str().is_empty() {
+            let chainspec_loader = ChainspecLoader {
+                chainspec,
+                root_dir,
+                reactor_exit: Some(ReactorExit::ProcessShouldExit(ExitCode::Abort)),
+                initial_state_root_hash: Digest::default(),
+                next_upgrade: None,
+                initial_block: None,
+                after_upgrade: false,
+            };
+            return (chainspec_loader, Effects::new());
+        }
 
         let next_upgrade = next_upgrade(root_dir.clone(), chainspec.protocol_config.version);
 
@@ -266,15 +280,16 @@ impl ChainspecLoader {
             initial_state_root_hash: Digest::default(),
             next_upgrade,
             initial_block: None,
+            after_upgrade: false,
         };
 
         (chainspec_loader, effects)
     }
 
     /// This is a workaround while we have multiple reactors.  It should be used in the joiner and
-    /// validator reactors' constructors to start the recurring task of checking for upgrades.  The
-    /// recurring tasks of the previous reactors will be cancelled when the relevant reactor is
-    /// destroyed during transition.
+    /// participating reactors' constructors to start the recurring task of checking for upgrades.
+    /// The recurring tasks of the previous reactors will be cancelled when the relevant reactor
+    /// is destroyed during transition.
     pub(crate) fn start_checking_for_upgrades<REv>(
         &self,
         effect_builder: EffectBuilder<REv>,
@@ -289,6 +304,12 @@ impl ChainspecLoader {
         self.reactor_exit
     }
 
+    /// Returns whether the current node instance is started immediately after an upgrade –
+    /// i.e. whether the last/highest block stored is a block that triggerred the upgrade.
+    pub(crate) fn after_upgrade(&self) -> bool {
+        self.after_upgrade
+    }
+
     /// The state root hash with which this session is starting.  It will be the result of running
     /// `ContractRuntime::commit_genesis()` or `ContractRuntime::upgrade()` or else the state root
     /// hash specified in the highest block.
@@ -312,10 +333,6 @@ impl ChainspecLoader {
         self.initial_block.as_ref()
     }
 
-    pub(crate) fn initial_block_hash(&self) -> Option<BlockHash> {
-        self.initial_block_header().map(|hdr| hdr.hash())
-    }
-
     /// This returns the era at which we will be starting the operation, assuming the highest known
     /// block is the last one. It will return the era of the highest known block, unless it is a
     /// switch block, in which case it returns the successor to the era of the highest known block.
@@ -329,7 +346,7 @@ impl ChainspecLoader {
     }
 
     /// Returns the era ID of where we should reset back to.  This means stored blocks in that and
-    /// subsequent eras are ignored (conceptually deleted from storage).
+    /// subsequent eras are deleted from storage.
     pub(crate) fn hard_reset_to_start_of_era(&self) -> Option<EraId> {
         self.chainspec
             .protocol_config
@@ -383,6 +400,7 @@ impl ChainspecLoader {
                 trace!("valid run immediately after upgrade");
                 let upgrade_config =
                     self.new_upgrade_config(&highest_block, previous_protocol_version);
+                self.after_upgrade = true;
                 return effect_builder
                     .upgrade_contract_runtime(upgrade_config)
                     .event(Event::UpgradeResult);
@@ -393,7 +411,11 @@ impl ChainspecLoader {
                 // TODO - if migrating data yields a new empty block as a means to store the
                 //        post-migration global state hash, we'll come to this code branch, and we
                 //        should not exit the process in that case.
-                warn!("invalid run, expected highest block to be switch block: exit to downgrade");
+                warn!(
+                    %current_chainspec_activation_point,
+                    %highest_block,
+                    "invalid run, expected highest block to be switch block: exit to downgrade"
+                );
                 self.reactor_exit =
                     Some(ReactorExit::ProcessShouldExit(ExitCode::DowngradeVersion));
                 return Effects::new();
@@ -403,43 +425,54 @@ impl ChainspecLoader {
         if highest_block_era_id < current_chainspec_activation_point {
             // This is an invalid run where blocks are missing from storage.  Try exiting the
             // process and downgrading the version to recover the missing blocks.
-            warn!("invalid run, missing blocks from storage: exit to downgrade");
+            warn!(
+                %current_chainspec_activation_point,
+                %highest_block,
+                "invalid run, missing blocks from storage: exit to downgrade"
+            );
             self.reactor_exit = Some(ReactorExit::ProcessShouldExit(ExitCode::DowngradeVersion));
             return Effects::new();
         }
 
-        let debug_assert_version_match = || {
-            debug_assert!(previous_protocol_version == self.chainspec.protocol_config.version);
+        let unplanned_shutdown = match self.next_upgrade {
+            Some(ref next_upgrade) => highest_block_era_id < next_upgrade.activation_point.era_id(),
+            None => true,
         };
 
-        let next_upgrade_activation_point = match self.next_upgrade {
-            Some(ref next_upgrade) => next_upgrade.activation_point.era_id(),
-            None => {
+        if unplanned_shutdown {
+            if previous_protocol_version == self.chainspec.protocol_config.version {
                 // This is a valid run, restarted after an unplanned shutdown.
-                debug_assert_version_match();
                 self.initial_state_root_hash = *highest_block.state_root_hash();
-                info!("valid run after an unplanned shutdown with no scheduled upgrade");
+                info!(
+                    %current_chainspec_activation_point,
+                    %highest_block,
+                    "valid run after an unplanned shutdown before upgrade due"
+                );
                 self.reactor_exit = Some(ReactorExit::ProcessShouldContinue);
-                return Effects::new();
+            } else {
+                // This is an invalid run, where we previously forked and missed an upgrade.
+                warn!(
+                    current_chainspec_protocol_version = %self.chainspec.protocol_config.version,
+                    %current_chainspec_activation_point,
+                    %highest_block,
+                    "invalid run after missing an upgrade and forking"
+                );
+                self.reactor_exit = Some(ReactorExit::ProcessShouldExit(ExitCode::Abort));
             }
-        };
-
-        if highest_block_era_id < next_upgrade_activation_point {
-            // This is a valid run, restarted after an unplanned shutdown.
-            debug_assert_version_match();
-            self.initial_state_root_hash = *highest_block.state_root_hash();
-            info!("valid run after an unplanned shutdown before upgrade due");
-            self.reactor_exit = Some(ReactorExit::ProcessShouldContinue);
             return Effects::new();
         }
 
-        // The is an invalid run as the highest block era ID >= next activation point, so we're
+        // This is an invalid run as the highest block era ID >= next activation point, so we're
         // running an outdated version.  Exit with success to indicate we should upgrade.
-        //
-        // TODO - once the block includes the protocol version, we can deduce here whether we're
-        //        running a version where we missed an upgrade and ran on a fork.  In that case, we
-        //        should set our exit code to `ExitCode::Abort`.
-        warn!("running outdated version: exit to upgrade");
+        warn!(
+            next_upgrade_activation_point = %self
+                .next_upgrade
+                .as_ref()
+                .map(|next_upgrade| next_upgrade.activation_point.era_id())
+                .unwrap_or_default(),
+            %highest_block,
+            "running outdated version: exit to upgrade"
+        );
         self.reactor_exit = Some(ReactorExit::ProcessShouldExit(ExitCode::Success));
         Effects::new()
     }
@@ -778,8 +811,17 @@ fn next_upgrade(dir: PathBuf, current_version: ProtocolVersion) -> Option<NextUp
 
 #[cfg(test)]
 mod tests {
+    use rand::Rng;
+
     use super::*;
-    use crate::{testing::TestRng, types::chainspec::CHAINSPEC_NAME};
+    use crate::{
+        logging,
+        reactor::{
+            participating::Event as ParticipatingEvent, EventQueueHandle, QueueKind, Scheduler,
+        },
+        testing::TestRng,
+        types::chainspec::CHAINSPEC_NAME,
+    };
 
     #[test]
     fn should_get_next_installed_version() {
@@ -963,4 +1005,360 @@ mod tests {
         fs::remove_file(&path_v1_0_0).unwrap();
         assert!(maybe_next_point(&current).is_none());
     }
+
+    struct TestFixture {
+        chainspec_loader: ChainspecLoader,
+        effect_builder: EffectBuilder<ParticipatingEvent>,
+    }
+
+    impl TestFixture {
+        fn new() -> Self {
+            let _ = logging::init();
+
+            // By default the local chainspec is a genesis one.  We don't want that for most tests,
+            // so set it to V1.5.0 activated at era 300.
+            let mut chainspec = Chainspec::from_resources("local");
+            chainspec.protocol_config.version = ProtocolVersion::from_parts(1, 5, 0);
+            chainspec.protocol_config.activation_point = ActivationPoint::EraId(EraId::new(300));
+
+            let chainspec_loader = ChainspecLoader {
+                chainspec: Arc::new(chainspec),
+                root_dir: PathBuf::from("."),
+                reactor_exit: None,
+                initial_state_root_hash: Digest::default(),
+                next_upgrade: None,
+                initial_block: None,
+                after_upgrade: false,
+            };
+
+            let scheduler = utils::leak(Scheduler::new(QueueKind::weights()));
+            let effect_builder = EffectBuilder::new(EventQueueHandle::new(scheduler));
+
+            TestFixture {
+                chainspec_loader,
+                effect_builder,
+            }
+        }
+
+        /// Returns the current chainspec's activation point.
+        fn current_activation_point(&self) -> EraId {
+            self.chainspec_loader
+                .chainspec
+                .protocol_config
+                .activation_point
+                .era_id()
+        }
+
+        /// Returns the current chainspec's protocol version.
+        fn current_protocol_version(&self) -> ProtocolVersion {
+            self.chainspec_loader.chainspec.protocol_config.version
+        }
+
+        /// Returns a protocol version earlier than the current chainspec's version.
+        fn earlier_protocol_version(&self) -> ProtocolVersion {
+            ProtocolVersion::from_parts(
+                self.current_protocol_version().value().major,
+                self.current_protocol_version().value().minor - 1,
+                0,
+            )
+        }
+
+        /// Returns a protocol version later than the current chainspec's version.
+        fn later_protocol_version(&self) -> ProtocolVersion {
+            ProtocolVersion::from_parts(
+                self.current_protocol_version().value().major,
+                self.current_protocol_version().value().minor + 1,
+                0,
+            )
+        }
+
+        /// Sets a valid value for the next upgrade in the chainspec loader.
+        fn set_next_upgrade(&mut self, era_diff: u64) {
+            self.chainspec_loader.next_upgrade = Some(NextUpgrade {
+                activation_point: ActivationPoint::EraId(
+                    self.current_activation_point() + era_diff,
+                ),
+                protocol_version: self.later_protocol_version(),
+            });
+        }
+
+        /// Calls `handle_initialize()` on the chainspec loader, asserting the provided block has
+        /// been recorded and that the expected number of effects were returned.
+        fn assert_handle_initialize(
+            &mut self,
+            maybe_highest_block: Option<Block>,
+            expected_effect_count: usize,
+        ) {
+            let effects = self.chainspec_loader.handle_initialize(
+                self.effect_builder,
+                maybe_highest_block.clone().map(Box::new),
+            );
+
+            assert_eq!(self.chainspec_loader.initial_block, maybe_highest_block);
+            assert_eq!(effects.len(), expected_effect_count);
+        }
+
+        /// Asserts that the chainspec loader indicates initialization is ongoing, i.e. that
+        /// `chainspec_loader.reactor_exit` is `None`.
+        fn assert_initialization_incomplete(&self) {
+            assert!(self.chainspec_loader.reactor_exit.is_none())
+        }
+
+        /// Asserts that the chainspec loader indicates initialization is complete and the node
+        /// process should not stop.
+        fn assert_process_should_continue(&self) {
+            assert_eq!(
+                self.chainspec_loader.reactor_exit,
+                Some(ReactorExit::ProcessShouldContinue)
+            )
+        }
+
+        /// Asserts that the chainspec loader indicates the process should stop to downgrade.
+        fn assert_process_should_downgrade(&self) {
+            assert_eq!(
+                self.chainspec_loader.reactor_exit,
+                Some(ReactorExit::ProcessShouldExit(ExitCode::DowngradeVersion))
+            )
+        }
+
+        /// Asserts that the chainspec loader indicates the process should stop to upgrade.
+        fn assert_process_should_upgrade(&self) {
+            assert_eq!(
+                self.chainspec_loader.reactor_exit,
+                Some(ReactorExit::ProcessShouldExit(ExitCode::Success))
+            )
+        }
+
+        /// Asserts that the chainspec loader indicates the process should stop with an error.
+        fn assert_process_should_abort(&self) {
+            assert_eq!(
+                self.chainspec_loader.reactor_exit,
+                Some(ReactorExit::ProcessShouldExit(ExitCode::Abort))
+            )
+        }
+    }
+
+    /// Simulates an initial run of the node where no blocks have been stored previously and the
+    /// chainspec is the genesis one.
+    #[test]
+    fn should_keep_running_if_first_run_at_genesis() {
+        let mut fixture = TestFixture::new();
+        fixture.chainspec_loader.chainspec = Arc::new(Chainspec::from_resources("local"));
+        assert!(fixture.chainspec_loader.chainspec.is_genesis());
+
+        // Should return a single effect (commit genesis).
+        fixture.assert_handle_initialize(None, 1);
+
+        // We're still waiting for the result of the commit genesis event.
+        fixture.assert_initialization_incomplete();
+    }
+
+    /// Simulates an initial run of the node where no blocks have been stored previously but the
+    /// chainspec is not the genesis one.
+    #[test]
+    fn should_downgrade_if_first_run_not_genesis() {
+        let mut fixture = TestFixture::new();
+        assert!(!fixture.chainspec_loader.chainspec.is_genesis());
+
+        fixture.assert_handle_initialize(None, 0);
+        fixture.assert_process_should_downgrade();
+    }
+
+    /// Simulates a valid run immediately after an upgrade.
+    #[test]
+    fn should_keep_running_after_upgrade() {
+        let mut fixture = TestFixture::new();
+        let mut rng = TestRng::new();
+
+        // Immediately after an upgrade, the highest block will be the switch block from the era
+        // immediately before the upgrade.
+        let previous_era = fixture.current_activation_point() - 1;
+        let height = rng.gen();
+        let earlier_version = fixture.earlier_protocol_version();
+        let highest_block =
+            Block::random_with_specifics(&mut rng, previous_era, height, earlier_version, true);
+
+        // Should return a single effect (commit upgrade).
+        fixture.assert_handle_initialize(Some(highest_block), 1);
+
+        // We're still waiting for the result of the commit upgrade event.
+        fixture.assert_initialization_incomplete();
+    }
+
+    /// Simulates an invalid run where the highest block is from the previous era, but isn't the
+    /// switch block.
+    ///
+    /// This is unlikely to happen unless a user modifies the launcher's config file to force the
+    /// wrong version of node to be executed, or somehow manually removes the last (switch) block
+    /// from storage as the node upgraded.
+    #[test]
+    fn should_downgrade_if_highest_block_is_from_previous_era_but_is_not_switch() {
+        let mut fixture = TestFixture::new();
+        let mut rng = TestRng::new();
+
+        // Make the highest block a non-switch block from the era immediately before the upgrade.
+        let previous_era = fixture.current_activation_point() - 1;
+        let height = rng.gen();
+        let earlier_version = fixture.earlier_protocol_version();
+        let highest_block =
+            Block::random_with_specifics(&mut rng, previous_era, height, earlier_version, false);
+
+        fixture.assert_handle_initialize(Some(highest_block), 0);
+        fixture.assert_process_should_downgrade();
+    }
+
+    /// Simulates an invalid run where the highest block is from an era before the previous era, and
+    /// may or may not be a switch block.
+    ///
+    /// This is unlikely to happen unless a user modifies the launcher's config file to force the
+    /// wrong version of node to be executed, or somehow manually removes later blocks from storage.
+    #[test]
+    fn should_downgrade_if_highest_block_is_from_earlier_era() {
+        let mut fixture = TestFixture::new();
+        let mut rng = TestRng::new();
+
+        // Make the highest block from an era before the one immediately before the upgrade.
+        let current_era = fixture.current_activation_point().value();
+        let previous_era = fixture.current_activation_point() - rng.gen_range(2..current_era);
+        let height = rng.gen();
+        let earlier_version = fixture.earlier_protocol_version();
+        let is_switch = rng.gen();
+        let highest_block = Block::random_with_specifics(
+            &mut rng,
+            previous_era,
+            height,
+            earlier_version,
+            is_switch,
+        );
+
+        fixture.assert_handle_initialize(Some(highest_block), 0);
+        fixture.assert_process_should_downgrade();
+    }
+
+    /// Simulates a valid run where the highest block is from an era the same or newer than the
+    /// current chainspec activation point, and there is no scheduled upcoming upgrade.
+    ///
+    /// This would happen in the case of an unplanned shutdown of the node.
+    #[test]
+    fn should_keep_running_if_unplanned_shutdown_and_no_upgrade_scheduled() {
+        let mut fixture = TestFixture::new();
+        let mut rng = TestRng::new();
+
+        // Make the highest block from an era the same or newer than the current chainspec one.
+        let future_era = fixture.current_activation_point() + rng.gen_range(0..3);
+        let height = rng.gen();
+        let current_version = fixture.current_protocol_version();
+        let is_switch = rng.gen();
+        let highest_block =
+            Block::random_with_specifics(&mut rng, future_era, height, current_version, is_switch);
+
+        fixture.assert_handle_initialize(Some(highest_block), 0);
+        fixture.assert_process_should_continue();
+    }
+
+    /// Simulates a valid run where the highest block is from an era the same or newer than the
+    /// current chainspec activation point, but older than a scheduled upgrade's activation point.
+    ///
+    /// This would happen in the case of an unplanned shutdown of the node.
+    #[test]
+    fn should_keep_running_if_unplanned_shutdown_and_future_upgrade_scheduled() {
+        let mut fixture = TestFixture::new();
+        let mut rng = TestRng::new();
+
+        // Set an upgrade for 10 eras after the current chainspec activation point.
+        let era_diff = 10;
+        fixture.set_next_upgrade(era_diff);
+
+        // Make the highest block from an era the same or newer than the current chainspec one.
+        let future_era = fixture.current_activation_point() + rng.gen_range(0..era_diff);
+        let height = rng.gen();
+        let current_version = fixture.current_protocol_version();
+        let is_switch = rng.gen();
+        let highest_block =
+            Block::random_with_specifics(&mut rng, future_era, height, current_version, is_switch);
+
+        fixture.assert_handle_initialize(Some(highest_block), 0);
+        fixture.assert_process_should_continue();
+    }
+
+    /// Simulates an invalid run where:
+    /// * the highest block is from an era the same or newer than the current chainspec activation
+    ///   point,
+    /// * there is no scheduled upcoming upgrade,
+    /// * and the protocol version of the highest block doesn't match the current chainspec version.
+    ///
+    /// This would happen in the case of an unplanned shutdown of the node after e.g. forking.
+    #[test]
+    fn should_abort_if_unplanned_shutdown_after_fork_and_no_upgrade_scheduled() {
+        let mut fixture = TestFixture::new();
+        let mut rng = TestRng::new();
+
+        // Make the highest block from an era the same or newer than the current chainspec one, but
+        // with an old protocol version.
+        let future_era = fixture.current_activation_point() + rng.gen_range(0..3);
+        let height = rng.gen();
+        let earlier_version = fixture.earlier_protocol_version();
+        let is_switch = rng.gen();
+        let highest_block =
+            Block::random_with_specifics(&mut rng, future_era, height, earlier_version, is_switch);
+
+        fixture.assert_handle_initialize(Some(highest_block), 0);
+        fixture.assert_process_should_abort();
+    }
+
+    /// Simulates an invalid run where:
+    /// * the highest block is from an era the same or newer than the current chainspec activation
+    ///   point,
+    /// * but older than a scheduled upgrade's activation point,
+    /// * and the protocol version of the highest block doesn't match the current chainspec version.
+    ///
+    /// This would happen in the case of an unplanned shutdown of the node after e.g. forking.
+    #[test]
+    fn should_abort_if_unplanned_shutdown_after_fork_and_future_upgrade_scheduled() {
+        let mut fixture = TestFixture::new();
+        let mut rng = TestRng::new();
+
+        // Set an upgrade for 10 eras after the current chainspec activation point.
+        let era_diff = 10;
+        fixture.set_next_upgrade(era_diff);
+
+        // Make the highest block from an era the same or newer than the current chainspec one, but
+        // with an old protocol version.
+        let future_era = fixture.current_activation_point() + rng.gen_range(0..era_diff);
+        let height = rng.gen();
+        let earlier_version = fixture.earlier_protocol_version();
+        let is_switch = rng.gen();
+        let highest_block =
+            Block::random_with_specifics(&mut rng, future_era, height, earlier_version, is_switch);
+
+        fixture.assert_handle_initialize(Some(highest_block), 0);
+        fixture.assert_process_should_abort();
+    }
+
+    /// Simulates an invalid run where the highest block is from an era the same or newer than the
+    /// than a scheduled upgrade's activation point.
+    ///
+    /// This is unlikely to happen unless a user modifies the launcher's config file to force the
+    /// wrong version of node to be executed.
+    #[test]
+    fn should_upgrade_if_highest_block_from_after_future_upgrade_activation_point() {
+        let mut fixture = TestFixture::new();
+        let mut rng = TestRng::new();
+
+        // Set an upgrade for 10 eras after the current chainspec activation point.
+        let era_diff = 10;
+        fixture.set_next_upgrade(era_diff);
+
+        // Make the highest block from an era the same or later than the upgrade activation point.
+        let future_era =
+            fixture.current_activation_point() + rng.gen_range(era_diff..(era_diff + 10));
+        let height = rng.gen();
+        let later_version = fixture.later_protocol_version();
+        let is_switch = rng.gen();
+        let highest_block =
+            Block::random_with_specifics(&mut rng, future_era, height, later_version, is_switch);
+
+        fixture.assert_handle_initialize(Some(highest_block), 0);
+        fixture.assert_process_should_upgrade();
+    }
 }
