@@ -35,6 +35,8 @@ fn new_config(harness: &ComponentHarness<UnitTestEvent>) -> Config {
         max_deploy_store_size: 50 * MIB,
         max_deploy_metadata_store_size: 50 * MIB,
         max_state_store_size: 50 * MIB,
+        enable_mem_deduplication: false,
+        mem_pool_prune_interval: 1024,
     }
 }
 
@@ -51,6 +53,7 @@ fn storage_fixture(harness: &ComponentHarness<UnitTestEvent>) -> Storage {
         &WithDir::new(harness.tmp.path(), cfg),
         None,
         ProtocolVersion::from_parts(1, 0, 0),
+        false,
     )
     .expect("could not create storage component fixture")
 }
@@ -71,6 +74,7 @@ fn storage_fixture_with_hard_reset(
         &WithDir::new(harness.tmp.path(), cfg),
         Some(reset_era_id),
         ProtocolVersion::from_parts(1, 1, 0),
+        false,
     )
     .expect("could not create storage component fixture")
 }
@@ -453,18 +457,21 @@ fn can_retrieve_block_by_height() {
         &mut harness.rng,
         EraId::new(1),
         33,
+        ProtocolVersion::V1_0_0,
         true,
     ));
     let block_14 = Box::new(Block::random_with_specifics(
         &mut harness.rng,
         EraId::new(1),
         14,
+        ProtocolVersion::V1_0_0,
         false,
     ));
     let block_99 = Box::new(Block::random_with_specifics(
         &mut harness.rng,
         EraId::new(2),
         99,
+        ProtocolVersion::V1_0_0,
         true,
     ));
 
@@ -578,12 +585,14 @@ fn different_block_at_height_is_fatal() {
         &mut harness.rng,
         EraId::new(1),
         44,
+        ProtocolVersion::V1_0_0,
         false,
     ));
     let block_44_b = Box::new(Block::random_with_specifics(
         &mut harness.rng,
         EraId::new(1),
         44,
+        ProtocolVersion::V1_0_0,
         false,
     ));
 
@@ -1043,6 +1052,7 @@ fn should_hard_reset() {
                 &mut harness.rng,
                 EraId::from(height as u64 / 3),
                 height as u64,
+                ProtocolVersion::V1_0_0,
                 is_switch,
             )
         })
