@@ -15,7 +15,6 @@ use crate::{
         chainspec_loader::{self, ChainspecLoader},
         contract_runtime::{self, ContractRuntime},
         gossiper,
-        network::NetworkIdentity,
         small_network::{GossipedAddress, SmallNetworkIdentity, SmallNetworkIdentityError},
         storage::{self, Storage},
         Component,
@@ -177,8 +176,6 @@ pub struct Reactor {
     pub(super) storage: Storage,
     pub(super) contract_runtime: ContractRuntime,
     pub(super) small_network_identity: SmallNetworkIdentity,
-    #[data_size(skip)]
-    pub(super) network_identity: NetworkIdentity,
 }
 
 impl Reactor {
@@ -232,15 +229,12 @@ impl Reactor {
 
         let small_network_identity = SmallNetworkIdentity::new()?;
 
-        let network_identity = NetworkIdentity::new();
-
         let reactor = Reactor {
             config,
             chainspec_loader,
             storage,
             contract_runtime,
             small_network_identity,
-            network_identity,
         };
         Ok((reactor, effects))
     }
@@ -309,11 +303,8 @@ impl reactor::Reactor for Reactor {
 #[cfg(test)]
 pub mod test {
     use super::*;
-    use crate::{
-        components::network::ENABLE_LIBP2P_NET_ENV_VAR, testing::network::NetworkedReactor,
-        types::Chainspec,
-    };
-    use std::{env, sync::Arc};
+    use crate::{testing::network::NetworkedReactor, types::Chainspec};
+    use std::sync::Arc;
 
     impl Reactor {
         pub(crate) fn new_with_chainspec(
@@ -332,11 +323,7 @@ pub mod test {
     impl NetworkedReactor for Reactor {
         type NodeId = NodeId;
         fn node_id(&self) -> Self::NodeId {
-            if env::var(ENABLE_LIBP2P_NET_ENV_VAR).is_err() {
-                NodeId::from(&self.small_network_identity)
-            } else {
-                NodeId::from(&self.network_identity)
-            }
+            NodeId::from(&self.small_network_identity)
         }
     }
 }
