@@ -1,7 +1,7 @@
 use thiserror::Error;
 
 use crate::{
-    components::{contract_runtime, network, small_network, storage},
+    components::{contract_runtime, small_network, storage},
     utils::ListeningError,
 };
 
@@ -12,10 +12,6 @@ pub enum Error {
     #[error("prometheus (metrics) error: {0}")]
     Metrics(#[from] prometheus::Error),
 
-    /// `Network` component error.
-    #[error("network error: {0}")]
-    Network(#[from] network::Error),
-
     /// `SmallNetwork` component error.
     #[error("small network error: {0}")]
     SmallNetwork(#[from] small_network::Error),
